#!/bin/sh
# File: host.sh
#
# Return system ID on stdout (all on one line) as:
#
# cpu:<cpu name> os:<os name> path:<path component> version:<n> \
# revision:<n> export:<n> dist:<name> dist_version:<n> dist_revision:<n>
#
# e.g., "cpu:I386 os:LINUX path:i386-linux version:2.4 \
#        revision:18 export:linux-x86 dist:RedHat dist_version:7 dist_revision:3\n"
#
# some valid cpu names are: sun3 sun4 alpha mips hp800 hp700 i386
# some valid OS names are: sunos solaris osf1 ultrix hp-ux linux
#
# Note: dist:<name>, dist_version:<n>, and dist_revision:<n>
# are generated only for known Linux systems.
#
# Update log
#
# Mar 21 2023 JVD: Added Rocky Linux.
# Jan 17 2023 JVD: Fixed to print "export:darwin-arm64".
# Dec  6 2022 NYP: Added Darwin-arm64
# Nov 10 2022 NYP: Added CentOS 8+
# Apr 27 2019 NYP: Added SUSE SLE 15
# Dec 14 2015 NYP: Port to ARM64. remove old cruft
# Apr 28 2015 JVD: Port to Linux-PowerLE (Power8+ Little-Endian).
# Apr 21 2015 JVD: Added ppc64le (powerle) on Linux.
# Mar 24 2014 NYP: Added Red Hat Enterprise Linux Workstation
# Oct 17 2013 NYP: Added Ubuntu distribution detection
# Mar 20 2013 NYP: Added host.sh -ver
# Feb 27 2013 NYP: Added k1om case as Intel MIC (Xeon Phi)
# Sep 23 2009 NYP: Added darwin-x86-64 as darwin-i386
# Apr 29 2009 JVD: Added mips64-linux.
# Apr 14 2009 NYP: Fixed darwin, that now has version 10.0.0
# Mar 30 2009 NYP: Added -arch option to get arch variable
# Feb 11 2009 JVD: Added CentOS, which is a RHEL derivative,
#		   thus the dist name "RedHatCentOS".
# Jan 20 2009 JVD: Added Red Hat Enterprise Linux Client.
# Jun  5 2008 JVD: Added Yellow Dog Linux, which is Fedora derivative which is
#		   sponsored by Red Hat, thus the dist name "RedHatYDL".
# Feb 29 2008 JVD: Added Fedora, without the "Core", like "Fedora release 7".
# Dec  7 2007 NYP: Added RHEL 5
# May 17 2007 RWG: Add -sh mode to output configuration variables in a format
#                  suitable for the sh eval command.
# Nov 28 2006 JVD: Set "dist_revision" to the RHEL "Update" number.
# Mar  3 2006 JVD: Use i386-darwin instead of x86-darwin.
# Feb  1 2005 RMM: Mac OS X: Corrected Darwin to seek i386 vice x86 in uname.
# Aug 12 2005 JVD: Mac OS X: Added Darwin-x86.
# May  4 2005 JVD: Mac OS X: use sw_vers to figure out the OS version.
# Dec 27 2004 JVD: Mac OS X Darwin-Power: Added mapping from Darwin version to Mac OS X version.
# Oct  1 2004 JVD: Distinguish between SLES and SuSE distribution names.
#		   Distinguish between RedHatFC and RedHat distribution names.
#		   Make darwin-power the export path for MAC OS X.
# Dec  1 2003 JVD: Added Power (ppc and ppc64) on Linux.
# Nov 12 2003 JVD: Added RedHatEL distribution name.
# Sep 18 2003 JVD: Added Linux distribution name for SuSE.
# Aug  7 2003 RWG: Add Mac OSX support
# Jul 11 2003 JVD: Added AMD x86_64 on Linux.
# Jun 26 2003 RWG: Add the Linux distribution name, version, and revision.
#                  Add support for RedHat initially. For some build decisions,
#                  the distribution version is more useful than the kernel
#                  version.
# Jun 17 2003 RTF: Added HPUX IA64.
# Jan 23 2003 RWG: Add a bit of extra code to distinguish between a
#                  stock RH 7.2 and RH 7.2 with the recommended updates
#                  to a 2.4-18 kernel (e.g. my home machine).
# Dec  7 2002 JVD: Set the export field to linux-ia64 instead of ia64-linux.
#		   This makes it consistent with the other Linux platform 
#                  export fields.
# Jul 12 2001 JVD: Recognize 32-bit "IRIX" machines.
# Jun 26 2001 JVD: Permit [BDESTVX] as verion type letter on Tru64.
# Aug 31 2000 JVD: Add the export: tagged value to spitout the platform tags
#		used by totalview historically.
#
#----------------------------------------------------------------------------
# $Id$

# pull indicators out of uname

system=`uname -s`
machine=`uname -m`
version=`uname -v`
revision=`uname -r`

dist_name=""
dist_version=""
dist_revision=""

v=0
r=0

# Prepend system with an X, in case uname wasn't found

case X${system} in
  XSunOS)
    case X$revision in
      X5*)
        case X$machine in
	  Xi86pc) os="SOLARIS"; p="sun5-x86" ;;
	  *)	 os="SOLARIS"; p="sun5" ;;
	esac
	;;
      *)
    esac 

    v=`expr $revision : "\([0-9]\)"`         # SunOS=4, Solaris=5
    r=`expr $revision : "[0-9]\.\([0-9]*\)"`
    ;;

  XHP-UX)    
    os="HPUX"
    v=`expr $revision : "B\.\([0-9][0-9]\)"`
    r=`expr $revision : "B\.[0-9][0-9]\.\([0-9]*\)"`

    case X$machine in
      Xia64)    p="ia64-hpux" ;;
      *)        p="hpux11-xxx" ;;
    esac
    ;;

  XAIX)
    os="AIX"
    p="rs6000"
    machine="RS6000" 
    v=${version}
    r=${revision}
    ;;

  XLynxOS)
   os="LYNXOS"
   case X${machine} in
    Xi386)    p="i386-lynx"  ;;
    XuSPARC1) p="sparc-lynx" ;;
    Xm68040)  p="m68k-lynx"  ;;
    XPowerPC) p="ppc-lynx"   ;;
    *)        p="x-lynx"     ;;
   esac
  ;;

  XUNIX_SV)
   os="UNIXWARE";
   case X${machine} in
     Xi386) p="i386-unixware" ;;
     *)     p="xxx-unixware" ;;
   esac
   ;;

  X*WIN32*)
    if [ "$NUTC" = "NUTC" ]; then 
        os="NUTC"
        p="nutc"
    else
        os="WIN32"
        p="win32"
    fi
    ;;

  XWindows_NT)    # uname under MKS
    if [ "$NUTC" = "NUTC" ]; then 
        os="NUTC"
        p="nutc"
    else
        os="WIN32"
        p="win32"
    fi
    ;;

  X*CYGWIN*)
    os="WIN32"
    p="win32"
    ;;

  XLinux)
    case X$machine in
      Xi*86)    p="i386-linux" ;;
      Xalpha)   p="alpha-linux" ;;
      Xia64)    p="ia64-linux" ;;
      Xx86_64)  p="x86-64-linux" ;;
      Xppc)     p="power-linux" ;;
      Xppc64)   p="power-linux" ;;
      Xppc64le) p="powerle-linux" ;;
      Xmips64)  p="mips64-linux" ;;
      Xk1om)    p="mic-linux" ;;
      Xaarch64) p="arm64-linux" ;;
      *)        p="xxx-linux" ;;
    esac
    os="LINUX"
    v=`expr $revision : "\([0-9]\.[0-9]\+\)"`      # kernel version (e.g. 2.0)
    r=`expr $revision : "[0-9]\.[0-9]\+\.\([0-9]*\)"`

    # for RedHat Linux, extract the distribution name and version

    if [ -f /etc/redhat-release ] ; then
      vers=`cat /etc/redhat-release`

      if [ `expr "$vers" : "Red Hat Enterprise Linux [AEW]S release "` != 0 ]; then
	dist_name="RedHatEL"
	vers=`expr "$vers" : "Red Hat Enterprise Linux [AEW]S release \([0-9]\+\(\(\.[0-9]\+\)\|\( (.* Update [0-9]\+)\)\)\?\)"`
      elif [ `expr "$vers" : "Fedora Core release "` != 0 ]; then
	dist_name="RedHatFC"
	vers=`expr "$vers" : "Fedora Core release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "Red Hat Linux release "` != 0 ]; then
	dist_name="RedHat"
	vers=`expr "$vers" : "Red Hat Linux release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "Red Hat Enterprise Linux release "` != 0 ]; then
	dist_name="RedHatEL"
	vers=`expr "$vers" : "Red Hat Enterprise Linux release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "Red Hat Enterprise Linux Server release "` != 0 ]; then
	dist_name="RedHatEL"
	vers=`expr "$vers" : "Red Hat Enterprise Linux Server release \([0-9]\+\(\(\.[0-9]\+\)\|\( (.* Update [0-9]\+)\)\)\?\)"`
      elif [ `expr "$vers" : "Red Hat Enterprise Linux Workstation release "` != 0 ]; then
	dist_name="RedHatEL"
	vers=`expr "$vers" : "Red Hat Enterprise Linux Workstation release \([0-9]\+\(\(\.[0-9]\+\)\|\( (.* Update [0-9]\+)\)\)\?\)"`
      elif [ `expr "$vers" : "Red Hat Enterprise Linux Client release "` != 0 ]; then
	dist_name="RedHatEL"
	vers=`expr "$vers" : "Red Hat Enterprise Linux Client release \([0-9]\+\(\(\.[0-9]\+\)\|\( (.* Update [0-9]\+)\)\)\?\)"`
      elif [ `expr "$vers" : "Fedora release "` != 0 ]; then
	dist_name="RedHatFC"
	vers=`expr "$vers" : "Fedora release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "Yellow Dog Linux release "` != 0 ]; then
	# Yellow Dog Linux is a derivative of the Red Hat sponsored Fedora project,
	# so technically speaking, it's not a Fedora distribution, but it does have
        # its root in Red Hat, so I (JVD) picked "RedHatYDL" as the distribution name.
	dist_name="RedHatYDL"
	vers=`expr "$vers" : "Yellow Dog Linux release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "CentOS release "` != 0 ]; then
	# CentOS is a derivative of Red Hat Enterprise Linux, so technically
	# speaking, it's not a Red Hat distribution, but it does have its root
	# in Red Hat, so I (JVD) picked "RedHatCentOS" as the distribution name.
	dist_name="RedHatCentOS"
	vers=`expr "$vers" : "CentOS release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "CentOS Linux release "` != 0 ]; then
	# CentOS is a derivative of Red Hat Enterprise Linux, so technically
	# speaking, it's not a Red Hat distribution, but it does have its root
	# in Red Hat, so I (JVD) picked "RedHatCentOS" as the distribution name.
	dist_name="RedHatCentOS"
	vers=`expr "$vers" : "CentOS Linux release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "CentOS Stream release "` != 0 ]; then
	# CentOS is a derivative of Red Hat Enterprise Linux, so technically
	# speaking, it's not a Red Hat distribution, but it does have its root
	# in Red Hat, so I (JVD) picked "RedHatCentOS" as the distribution name.
	dist_name="RedHatCentOS"
	vers=`expr "$vers" : "CentOS Stream release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      elif [ `expr "$vers" : "Rocky Linux release "` != 0 ]; then
	# Rocky Linux is a derivative of Red Hat Enterprise Linux, so technically
	# speaking, it's not a Red Hat distribution, but it does have its root
	# in Red Hat, so I (JVD) picked "RedHatRocky" as the distribution name.
	dist_name="RedHatRocky"
	vers=`expr "$vers" : "Rocky Linux release \([0-9]\+\(\.[0-9]\+\)\?\)"`
      else
	dist_name="RedHatUNKNOWN"
	vers="0.0"
      fi

      dist_version=`expr "$vers" : "\([0-9]\+\)"`
      if [ `expr "$vers" : "[0-9]\+ (.* Update [0-9]\+)"` -gt 0 ]; then
	dist_revision=`expr "$vers" : "[0-9]\+ (.* Update \([0-9]\+\))"`
      else
	dist_revision=`expr "$vers" : "[0-9]\+\.\([0-9]\+\)"`
      fi
      if [ -z "$dist_revision" ] ; then
        dist_revision="0"
      fi

      # The recommended patches update the RH  7.2 kernel to 2.14.18.
      # Some test predicates confuse this with RH 7.3. Tack -rh72 to r to
      # distinguish this special case

       if [ "$r" = "18" -a "$vers" = "7.2" ]; then
        r="$r-rh72"
      fi
    elif [ -f /etc/SuSE-release ] ; then
      if egrep -q '(Enterprise Server)|(SLES)' /etc/SuSE-release; then
	dist_name="SLES"
      else
	dist_name="SuSE"
      fi

      vers=`awk '/^VERSION/ { print $3 }' /etc/SuSE-release`

      dist_version=`expr "$vers" : "\([0-9]\+\)"`
      dist_revision=`expr "$vers" : "[0-9]\+\.\([0-9]\+\)"`
      if [ -z "$dist_revision" ] ; then
        dist_revision="0"
      fi
    elif [ -f /etc/SUSE-brand ] ; then
	if egrep -q 'SLE' /etc/SUSE-brand; then
	   dist_name="SLES"
        else
	   dist_name="SUSE-any"
        fi 
	dist_version=`awk '/^VERSION/ { print $3 }' /etc/SUSE-brand`
        dist_revision="0"

    elif [ -f /etc/lsb-release ] ; then
	vers=`cat /etc/lsb-release`
	if [ `expr "$vers" : "DISTRIB_ID=Ubuntu"` != 0 ]; then
	   dist_name="Ubuntu"
	   vers=`awk '/^DISTRIB_RELEASE/ { sub(/DISTRIB_RELEASE=/,""); print; }' /etc/lsb-release`

        fi
        dist_version=`expr "$vers" : "\([0-9]\+\)"`
	dist_revision=`expr "$vers" : "$dist_version.\([0-9]\+\)"`
    fi
    ;;

  XDarwin)
    os="DARWIN"
    case X${machine} in
     "XPower Macintosh") p="power-darwin" ;;
     "Xi386")            p="i386-darwin" ;;
     "Xx86_64")          p="i386-darwin" ;;
     "Xarm64")           p="arm64-darwin" ;;
      *)                 p="xxx-darwin"   ;;
    esac
    v=`expr $revision : "\([0-9]\{1,\}\)"`
    r=`expr $revision : "[0-9]\{1,\}\.\([0-9]*\)"`
    dist_name="MacOSX"
    dist_vers=`sw_vers -productVersion`
    dist_version=`expr "$dist_vers" : "\([0-9]\{1,\}\)"`
    dist_revision=`expr "$dist_vers" : "[0-9]\{1,\}\.\([0-9]\{1,\}\)"`
    ;;

  *)         os="UNKNOWN"; p="unknown" ;;
esac

# figure out machine type
case X${machine} in
    Xsun386)  cpu="SUN386"  ;;
    Xsun4*)   cpu="SUN4"    ;;
    Xi386)    cpu="I386"    ;;
    XuSPARC1) cpu="SPARC"   ;;
    XPentium) cpu="I386"    ;;
    XRS6000)  cpu="RS6000"  ;;
    XPowerPC) cpu="PPC"     ;;
    Xppc)     cpu="POWER"   ;;
    Xppc64)   cpu="POWER"   ;;
    Xppc64le) cpu="POWERLE" ;;
    Xi*86)    cpu="I386"    ;;
    X586)     cpu="I386"    ;;
    Xi86pc)   cpu="I386"    ;;
    Xx86)     cpu="I386"    ;;
    Xia64)    cpu="IA64"    ;;
    Xx86_64)  cpu="X86_64"  ;;
    Xmips64)  cpu="MIPS64"  ;;
    Xk1om)    cpu="MIC"     ;; 
    Xaarch64) cpu="ARM64"   ;; 
    Xarm64) cpu="ARM64"   ;; 
    *)        cpu="UNKNOWN" ;;
esac

# Map the path to the values TV's has used historically
case X${p} in
    Xi386-linux)	exp=linux-x86	;;
    Xia64-linux)	exp=linux-ia64	;;
    Xpower-linux)	exp=linux-power	;;
    Xpowerle-linux)	exp=linux-powerle ;;
    Xia64-hpux) 	exp=hpux11-ia64	;;
    Xx86-64-linux)	exp=linux-x86-64;;
    Xarm64-linux)	exp=linux-arm64 ;;
    Xpower-darwin)	exp=darwin-power;;
    Xi386-darwin)	exp=darwin-x86  ;;
    Xarm64-darwin)	exp=darwin-arm64  ;;
    Xmips64-linux)	exp=linux-mips64;;
    *)			exp="$p"	;;
esac

# Label the distribution info if any is available

if [ -n "$dist_name" ] ; then
  DIST_NAME="DIST_NAME=$dist_name"
  dist_name="dist:$dist_name"
fi

if [ -n "$dist_version" ] ; then
  DIST_VERSION="DIST_VERSION=$dist_version"
  dist_version="dist_version:$dist_version"
fi

if [ -n "$dist_revision" ] ; then
  DIST_REVISION="DIST_REVISION=$dist_revision"
  dist_revision="dist_revision:$dist_revision"
fi

# tell the caller

if [ "$1" = "-sh" ] ; then
  major=`expr $v : '\(.*\)\..*'`
  minor=`expr $v : '.*\.\(.*\)'`
  echo "CPU=$cpu OS=$os OS_PATH=$p OS_VERSION=$v OS_REVISION=$r "\
    "OS_EXPORT_PATH=$exp $DIST_NAME $DIST_VERSION $DIST_REVISION "\
    "OS_VERSION_MAJOR=$major OS_VERSION_MINOR=$minor"
elif [ "$1" = "-path" ] ; then
    echo $p
elif [ "$1" = "-arch" ] ; then
    echo $exp
elif [ "$1" = "-ver" ] ; then
    echo $v
else
  echo cpu:$cpu os:$os path:$p version:$v revision:$r export:$exp \
    $dist_name $dist_version $dist_revision
fi
